#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "gfx.h"
#include "pngio.h"

#define PAD 10
#define SCRW 800
#define SCRH 600
#define WIDH 30

#define  NPIX    32
#define  PIXSIZE 16
#define  AREA_W  NPIX*PIXSIZE
#define  AREA_H  NPIX*PIXSIZE

#define SHOW_W  NPIX
#define SHOW_H  NPIX

#define  AREA_X PAD
#define  AREA_Y PAD

#define  SHOW_X (AREA_X+AREA_W+PAD)
#define  SHOW_Y AREA_Y
#define  BUT2_X PAD 
#define  ENT_X  (BUT2_X+BUT_W+PAD)
#define  BUT_W  80
#define  BUT_X  (AREA_X+AREA_W-BUT_W)
#define  ENT_W  (BUT_X-PAD-ENT_X)
#define  BUT_H  30
#define  ENT_H  BUT_H
#define  ENT_Y  (AREA_Y+AREA_H+PAD)
#define  BUT_Y  ENT_Y

static unsigned int pixbuf[NPIX*NPIX];
static unsigned int bigbuf[NPIX*NPIX*PIXSIZE*PIXSIZE];

void set_pixel(int x,int y, int color)
{
  if (x<0 || y<0 || y>=NPIX || x>=NPIX) return ;
  pixbuf[x+y*NPIX]= color | 0xff000000;
}

unsigned int get_pixel(int x,int y)
{
  if (x<0 || y<0 || y>=NPIX || x>=NPIX) return 0xff000000;
  return pixbuf[x+y*NPIX];
}

void draw()
{
  int x,y,xp,yp,P;
  for(x=0;x<NPIX;x++)
   for(y=0;y<NPIX;y++)
    { P= pixbuf[y*NPIX+x];
      for(xp=0;xp<PIXSIZE;xp++)
       for(yp=0;yp<PIXSIZE;yp++)
        bigbuf[ (yp+y*PIXSIZE)*(NPIX*PIXSIZE) + (xp+x*PIXSIZE) ]= P;
     }
  for(x=1;x<NPIX;x++)
    for(y=0;y<NPIX*PIXSIZE;y++)
      bigbuf[ y*(NPIX*PIXSIZE) + (x*PIXSIZE) ]= (x%5==0) ? 0xffffff: 0x888888;
  for(y=1;y<NPIX;y++)
    for(x=0;x<NPIX*PIXSIZE;x++)
      bigbuf[ y*PIXSIZE*NPIX*PIXSIZE + x ] = (y%5==0) ? 0xffffff : 0x888888;

  gfx_put(GFX_UPDATE,bigbuf,AREA_W,AREA_H, 0,0, AREA_X, AREA_Y,
            AREA_W, AREA_H);
  gfx_put(GFX_UPDATE,pixbuf, NPIX, NPIX, 0,0, SHOW_X, SHOW_Y, NPIX,NPIX);
}

void pixinit(int *x,int* y)
{
  int i;
  if (gfx_init(NULL, 800,600, "Pixel Edit"))
    exit(0);
  gfx_fill_rect(GFX_UPDATE,0,0,SCRW,SCRH,0);
  gfx_text_entry(GFX_UPDATE|GFX_FOCUS, ENT_X, ENT_Y, ENT_W, ENT_H, "");
  gfx_button(GFX_UPDATE, BUT_X, BUT_Y, BUT_W, BUT_H, "Save", GFXEV_LAST+1);
  gfx_button(GFX_UPDATE, BUT2_X, BUT_Y, BUT_W, BUT_H, "Load", GFXEV_LAST+2);
  gfx_draw_rect(GFX_UPDATE, AREA_X-1, AREA_Y-1, AREA_W+2, AREA_H+2,0xffff00);
  gfx_draw_rect(GFX_UPDATE, SHOW_X-1, SHOW_Y-1, SHOW_W+2, SHOW_W+2,0xffff00);
  for(i=0;i<NPIX*NPIX;i++) pixbuf[i]= 0xff000000;
  draw();
  *x= SHOW_X;
  *y= SHOW_Y + SHOW_H + PAD;
}

void load_image(char *fn)
{
   char *e; unsigned int iw,ih, *id,x,y;
   printf("load from %s\n",fn);
   e= read_png_image(fn, &iw, &ih, &id);
   if (e) { printf("error reading %s: %s\n",fn,e); return; }
   for(x=0;x<NPIX && x<iw;x++)
     for(y=0;y<NPIX && y<ih;y++)
       set_pixel(x,y, id[x+y*iw]);
   free(id);
}

void pixevent(gfx_event_t *ev)
{
  char *fn,*e;
  while(1)
    if (gfx_event(NULL,ev))
      switch(ev->type)
      {
        case GFXEV_LAST+1: 
         fn= gfx_text_entry_get(0);
         if (fn[0]==0) continue;
         printf("save to %s\n",fn);
         e= write_png_image(fn, NPIX, NPIX, pixbuf);
         if (e) printf("error writing %s: %s\n",fn,e);
         break;
        case GFXEV_LAST+2:
         fn= gfx_text_entry_get(0);
         if (fn[0]==0) continue;
         load_image(fn);
         draw();
         break;
        case GFXEV_BUTTON_PRESS:
         if (ev->button!=1) continue;
         if (ev->x<AREA_X || ev->x>=(AREA_X+AREA_W) ||
             ev->y<AREA_Y || ev->y>=(AREA_Y+AREA_H)) continue;
         ev->x= (ev->x-AREA_X)/PIXSIZE;
         ev->y= (ev->y-AREA_Y)/PIXSIZE;
         return ;
        default: return ;
      }
}

