#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "gfx.h"

#define THR 1.8

#define NPIX 32

int GA_x, GA_y;

void button(char *name, int action)
{
  gfx_button(GFX_UPDATE, GA_x, GA_y, 80, 30, name, action);
  GA_y+= 40;
}

unsigned int *copy()
{
  unsigned int *T;
  int x,y;
  T= malloc(NPIX*NPIX*sizeof(unsigned int));
  for(x=0;x<NPIX;x++)
    for(y=0;y<NPIX;y++)
      T[x+y*NPIX]= get_pixel(x,y);
  return T;
}

int label(char *title,char *text)
{
  int w,h,r;
  gfx_print(GFX_UPDATE, 1, 0xffffff, 0,GA_x, GA_y+5, title, &w, &h); 
  r= gfx_label(GFX_UPDATE, 1, 0xffffff, 0, GA_x+w+5, GA_y, 150, h+10, text);
  GA_y+= h+10 + 10;
  return r;
}

int entry(char *name)
{
  int w,h,r;
  gfx_print(GFX_UPDATE, 1, 0xffffff, 0,GA_x, GA_y+5, name, &w, &h); 
  r= gfx_text_entry(GFX_UPDATE, GA_x+w+5, GA_y, 120, h+10, "");
  GA_y+= h+10 + 10;
  return r;
}

int color(int e)
{
  char *K;
  char *P;
  int R,G,B;
  K= gfx_text_entry_get(e);
  P= strchr(K,',');
  if (!P)
    return strtol(K, NULL, 16);
  P[0]= 0;

  R= atoi(K)&0xff;
  K= P+1;
  if (K[0]==0) return R<<16;
  P= strchr(K,',');
  if (!P)
  {
    G= atoi(K) & 0xff;
    return (R<<16) | (G<<8);
  }
  P[0]= 0;
  G= atoi(K) & 0xff;
  K= P+1;
  B= atoi(K) & 0xff;
  return (R<<16) | (G<<8) | B;
}

void point(int x,int y, int color)
{
  set_pixel(x,y,color);
  draw();
}

unsigned int BL(int A, double V, int B)
{
  int K;
  K= A*V + (1.0-V)*B;
  if (K<0) K= 0; if (K>255) K= 255;
  return K;
}

void line(int x0,int y0,int x1,int y1, int col1, int col2)
{
  int x,y,sigx,sigy,i,len; double V;
  if (abs(x0-x1)>abs(y0-y1)) y1= y0;
                       else  x1= x0;
  len= abs(x1-x0) + abs(y0-y1);
  if (len==0) return ;
  sigx= (x1-x0) /len; 
  sigy= (y1-y0) /len; 
  for(i=0;i<=len;i++)
  {
    x= x0+ sigx*i;
    y= y0+ sigy*i;
    V= 1.0 - (double) i / len;

    set_pixel(x,y, (BL((col1 >> 16)&0xff, V, (col2>>16)&0xff) << 16 ) |
                   (BL((col1 >> 8 )&0xff, V, (col2>> 8)&0xff) << 8 ) |
                    BL((col1      )&0xff, V, (col2    )&0xff) );
  }
  draw();
}

int main(int argc,char** argv)
{
  int c1,c2,l1;
  int PX[2], PY[2];
  int np= 0;
  int F;
  int x,y;
  unsigned int *T;

  pixinit(&GA_x, &GA_y);
  c1= entry("Color1");
  c2= entry("Color2");
  l1= label("Function: ","Point");
  button("Point", GFXEV_LAST+4);
  button("Line", GFXEV_LAST+3);
  label("Move Image","");
  button("Up", GFXEV_LAST+5);
  button("Left", GFXEV_LAST+6);
  button("Rotate", GFXEV_LAST+7);
  button("Flip X", GFXEV_LAST+8);
  button("Flip Y", GFXEV_LAST+9);
  button("Invert", GFXEV_LAST+10);
  F= 'P';

  if (argc>1) { load_image(argv[1]); gfx_text_entry_set(1,0,argv[1]); draw(); }
  while(1)
  {
    gfx_event_t event;
    pixevent(&event);
    switch(event.type)
    {
    case GFXEV_QUIT: goto done;
    case GFXEV_BUTTON_PRESS: 
      PX[np]= event.x; PY[np]= event.y;
      if (F=='P') { point(PX[0], PY[0], color(c1)); np= 0; continue; }
      if (F=='L') { np++; if (np!=2) continue;
                    line(PX[0],PY[0], PX[1], PY[1], color(c1), color(c2));
                    np= 0; continue; }
      printf("pixel %d %d\n",event.x,event.y); break;
      break;
     case GFXEV_LAST+4:
       F= 'P'; np=0; gfx_label_set_text(GFX_UPDATE,l1,"Point"); break;
     case GFXEV_LAST+3:
       F= 'L'; np=0; gfx_label_set_text(GFX_UPDATE,l1,"Line"); break;
     case GFXEV_LAST+5:
       for(y=0;y<NPIX-1;y++)
         for(x=0;x<NPIX;x++)
           set_pixel(x,y, get_pixel(x,y+1));
       for(x=0;x<NPIX;x++) set_pixel(x, NPIX-1,0);
       draw();
       break;
     case GFXEV_LAST+6:
       for(x=0;x<NPIX-1;x++)
         for(y=0;y<NPIX;y++)
           set_pixel(x,y, get_pixel(x+1,y));
       for(y=0;y<NPIX;y++) set_pixel(NPIX-1, y,0);
       draw();
       break;
     case GFXEV_LAST+7:
         T= copy();
         for(y=0;y<NPIX;y++)
           for(x=0;x<NPIX;x++)
             set_pixel(NPIX-1-y,x, T[x+y*NPIX]);
         free(T); draw();
         break;
     case GFXEV_LAST+8:
         T= copy();
         for(y=0;y<NPIX;y++)
           for(x=0;x<NPIX;x++)
             set_pixel(NPIX-1-x, y, T[x+y*NPIX]);
         free(T); draw();
         break;
     case GFXEV_LAST+9:
         T= copy();
         for(y=0;y<NPIX;y++)
           for(x=0;x<NPIX;x++)
             set_pixel(x, NPIX-1-y, T[x+y*NPIX]);
         free(T); draw();
         break;
      case GFXEV_LAST+10:
         for(y=0;y<NPIX;y++)
           for(x=0;x<NPIX;x++)
             set_pixel(x, y, get_pixel(x,y) ^ 0xffffff);
         draw();
         break;
    }
  }
done:
  gfx_close();
  return 0;
}

